<?php
#---------------------------------------------------------------------------------------------------
#   Twitterタイムライン表示ブログパーツ Vers 0.41
#---------------------------------------------------------------------------------------------------
#
#   created by ymch
#   http://ymch.jp/
#
#   usage:
#       <div style="width:180px; height:170px; overflow:auto;">
#       <script type="text/javascript" src="http:/[yoursite]/twitter/" charset="utf-8"></script>
#       </div>
#
#   キャッシュの有効時間は60秒未満にしないほう良い。
#   （APIへのGETリクエスト上限は15分に15件まで。超過すると規制される。）

#   【注意点】
#       ・cURLを使うので「--with-curl」付きでコンパイルしてないPHPでは動作しません。
#       ・昔と違って、最近は端末が高性能。それ故、処理の殆どをJavaScriptにしてしまった。
#       ・それにPHPのjson_parseは結構サーバーのリソース喰うのよ。
#       ・ほとんど思い付きだけで書いてる。ソースの見直しはスゴく必要かも。

//-------------------------------------------------------------------------------------------------
// CONFIG
//-------------------------------------------------------------------------------------------------

define('SITE_URL', 'http://hoge.jp/');  /* サイトURL（末尾は「/」で）                           */
define('USER_ID', 'screen_name');       /* あなたのTwitterユーザーID                            */
define('CACHE_DIR', './cache/');        /* キャッシュディレクトリ（パーミッションは707に）      */
define('CACHE_EXPIRE', 60);             /* キャッシュの有効時間（秒）                           */
define('GET_TWEET_CNT', 50);            /* 取得ツイート数                                       */
define('GET_RETWEET' , true);           /* リツイートを拾うか否か（true | false）               */
define('TWITTER_API', 'https://api.twitter.com');

// 以下はアプリ登録 http://twitter.com/oauth_clients アプリ登録してアクセストークンを作成。
// そのあと http://dev.twitter.com/apps/ で該当アプリの「OAuth tool」に行けば一通り揃うハズ。

// Consumer key
define('TWTR_CONSUMER_KEY',    '**********************');

// Consumer secret
define('TWTR_CONSUMER_SECRET', '*****************************************');

// Access Token (oauth_token)
define('TWTR_ACCESS_TOKEN',      '**************************************************');

// Access Token Secret (oauth_token_secret)
define('TWTR_ACCESS_SECRET',   '******************************************');

//-------------------------------------------------------------------------------------------------

// 処理実行
createOutput();

////////////////////////////////////////////////////////////////////////////////////////////////////

function createOutput() {
/*
 * 【出力とキャッシュ生成】
 *
 *      引数：なし
 *      戻値：なし
 *
 */

    $json = "";
    $output = "";

    // ヒアドキュメントの中で利用するので変数化しておく
    $SITE_URL = SITE_URL;
    $USER_ID = USER_ID;

    // 該当キャッシュのファイルパス
    $cacheFile = CACHE_DIR . "cache";

    // キャッシュファイルが無ければ作成する
    if(! file_exists($cacheFile)){
        touch($cacheFile);
        chmod($cacheFile, 0666);
    }

    // キャッシュの有効期限チェック
    $cacheStamp = @filemtime($cacheFile);

    // 有効期限切れ、あるいはキャッシュが無い場合
    if (($cacheStamp < time() - CACHE_EXPIRE) || (! $cacheStamp)) {
        // 最新ステータスを得る
        $json = getStatuses();

        // キャッシュを作成
        $fp = fopen($cacheFile, "w");
        fwrite($fp, $json);
        fclose($fp);
    }
    // 有効期限内であればキャッシュを読み込んで出力
    else {
        $fp = fopen($cacheFile, "r");
        $json = fread($fp, filesize($cacheFile));
        fclose($fp);
    }

    $output = <<<_EOM_
document.write('<dl>');

// TwitterAPIから受け取ったJSON…というかオブジェクト
var jsonArray = {$json};

for (var i = 0; i < jsonArray.length; i++) {
    var obj = jsonArray[i];

    /*
     * ツイート本文の加工
     */

    // ツイート本文
    var tweet = obj.text;

    // 改行
    tweet = tweet.replace(/\\n/g, '<br />');

    // Twitter公式のサムネイル挿入
    if (obj.entities.media) {
        if (obj.entities.media[0].type == "photo") {
            tweet = tweet.replace(obj.entities.media[0].url,
                '<br /><a href="'
                + obj.entities.media[0].media_url + ':large'
                + '" target="_blank"><img src="'
                + obj.entities.media[0].media_url + ':thumb'
                + '" /></a>');
        }
    }

    // 一般URL短縮「t.co」を展開＆リンク
    var urlCnt = obj.entities.urls.length;
    if (urlCnt > 0) {
        for (var j = 0; j < urlCnt; j++) {
            tweet = tweet.replace(obj.entities.urls[j].url,
            '<a href="'
            + obj.entities.urls[j].expanded_url
            + '" target="_blank">'
            + obj.entities.urls[j].expanded_url
            + '</a>');
        }
    }

    // instagramのサムネイル挿入＆リンク処理
    tweet = tweet.replace(/\"_blank\">https:\/\/instagram[.]com\/p\/([\w\-]+)\/<\/a>/g,
          '"_blank"><br /><img src="https://instagram.com/p/'
          + "$1"
          + '/media/?size=t" width="150" height="150" alt=""></a>');

    // Twitpicのサムネイル挿入＆リンク処理
    tweet = tweet.replace(/\"_blank\">http:\/\/twitpic[.]com\/(\w+)<\/a>/g,
        '"_blank"><br /><img src="http://twitpic.com/show/thumb/'
        + "$1"
        + '" width="150" height="150" alt=""></a>');

    // フォト蔵のサムネイル挿入＆リンク処理（
    tweet = tweet.replace(/\"_blank\">http:\/\/photozou[.]jp\/photo\/show\/([0-9]+)\/([0-9]+)<\/a>/g,
        '"_blank"><br /><img src="http://photozou.jp/p/thumb/'
        + "$2"
        + '" width="150" height="150" alt=""></a>');

    // yFlogのサムネイル挿入＆リンク処理
    tweet = tweet.replace(/\"_blank\">http:\/\/yfrog[.]com\/(\w+)<\/a>/g,
        '"_blank"><br /><img src="http://yfrog.com/'
        + "$1"
        + '.th.jpg" width="150" alt=""></a>');

    // twitvideoのサムネイル挿入＆リンク処理
    tweet = tweet.replace(/\"_blank\">http:\/\/twitvideo[.]jp\/(\w+)<\/a>/g,
        '"_blank"><br /><img src="http://twitvideo.jp/img/thumb/'
        + "$1"
        + '" alt=""></a>');

    // 携帯百景のリンク処理
    tweet = tweet.replace(/\"_blank\">http:\/\/movapic[.]com\/pic\/(\w+)<\/a>/g,
        '"_blank"><img src="http://image.movapic.com/pic/t_'
        + "$1"
        + '.jpeg" alt=""></a>');

    // リツイートを自分のツイートと明確に区別
    if (obj.retweeted_status) {
        var retweet_ref_name = obj.retweeted_status.user.name;
        retweet_ref_url =
        tweet = tweet.replace(/^RT @([\w]+):/g,
            '【<a href="http://twitter.com/'
            + "$1"
            + '" target="_blank">'
            + retweet_ref_name
            + '</a>さんのTweet紹介】<br />');
    }

    // 宛先のリンク
    var mentionCnt = obj.entities.user_mentions.length;
    if (mentionCnt > 0) {
        for (var j = 0; j < mentionCnt; j++) {
            tweet = tweet.replace('@' + obj.entities.user_mentions[j].screen_name,
                '@<a href="http://twitter.com/'
                + obj.entities.user_mentions[j].screen_name
                + '" title="'
                + obj.entities.user_mentions[j].name
                + '" target="_blank">'
                + obj.entities.user_mentions[j].screen_name
                + '</a>');
        }
    }

    // ハッシュタグをリンク
    var hashtagCnt = obj.entities.hashtags.length;
    if (hashtagCnt > 0) {
        for (var j = 0; j < hashtagCnt; j++) {
            tweet = tweet.replace('#' + obj.entities.hashtags[j].text,
            '#<a href="https://twitter.com/search?q=%23'
            + encodeURI(obj.entities.hashtags[j].text)
            + '&src=hash" target="_blank">'
            + obj.entities.hashtags[j].text
            + '</a>');
        }
    }

    // 本文が 0x5C で終わると「\'」→ unterminated string literal（閉じてないことになるので）
    // 0x5C で終わるのはキャラクタ2バイト目が 0x5C 、すなわち 0x5C の重複ケース
    tweet = tweet.replace("/\\\\\\\\$/", "\\\\ ");

    // JavaScriptで￥が消える件
    tweet = tweet.replace('\\\\', '\\\\\\\\');

    // シングルクォートはJavaScriptの括りを破壊してしまうので
    tweet = tweet.replace("/'/", "\'");

    /*
     * ツイート本文の出力
     */
    document.write('<dt>');
    document.write(tweet);
    document.write('</dt>');

    /*
     * 付帯情報の組み立て
     */

    // ツイート本体のURL
    tweetUrl = 'http://twitter.com/'
             + '{$USER_ID}'
             + '/statuses/'
             + obj.id_str;

    // 返信ツイートアイコン
    var replyTo = "";
    if (obj.in_reply_to_status_id_str) {
        replyTo = '<a href="http://twitter.com/'
                + obj.in_reply_to_screen_name
                + '/statuses/'
                + obj.in_reply_to_status_id_str
                + '"'
                + ' target="_blank">'
                + '<img src="' + '{$SITE_URL}' + 'twitter/reply.png" width="12" height="14"'
                + ' alt="返信元ツイートを見る" title="返信元ツイートを見る" border="0"'
                + ' style="vertical-align:middle;" /></a>' + ' ';
    }

    // 位置情報アイコン
    var point = "";
    if (obj.geo) {
        if (obj.geo.coordinates.length) {
            var lat = obj.geo.coordinates[0];
            var lng = obj.geo.coordinates[1];
            point = '<a href="http://maps.google.com/maps?q='
                  + lat + ',' + lng
                  + '" target="_blank">'
                  + '<img src="' + '{$SITE_URL}' + 'twitter/pin.png" width="9" height="14"'
                  + ' alt="位置情報" title="位置情報" border="0" style="vertical-align:middle;" /></a>&nbsp;';
        }
    }

    // ツイート日時
    var dateValues = obj.created_at.split(" ");
    dateValue = dateValues[1] + " " + dateValues[2] + ", " + dateValues[5] + " " + dateValues[3];
    var date = new Date(dateValue);
    date.setHours(date.getHours() + 9);
    var year = date.getYear();
    if (year < 2000) { year += 1900; }
    var mon = '0' + (date.getMonth() + 1);
    mon = mon.substr(mon.length - 2);
    var day = '0' + date.getDate();
    day = day.substr(day.length - 2);
    var dy = date.getDay();
    dy = new Array("(日)","(月)","(火)","(水)","(木)","(金)","(土)")[dy];
    var hour = '0' + date.getHours();
    hour = hour.substr(hour.length - 2);
    var minute = '0' + date.getMinutes();
    minute = minute.substr(minute.length - 2);
    var dateText = year + "/" + mon + "/" + day + dy + " " + hour + ":" + minute;

    // リツイートされた件数
    var rt_stat = "";
    if (obj.retweet_count) {
        rt_stat = '<img src="' + '{$SITE_URL}' + 'twitter/retweet.png" width="18" height="14"'
                + ' alt="'
                + obj.retweet_count
                + '人からリツイートされてる" title="'
                + obj.retweet_count
                + '人からリツイートされてる"'
                + ' border="0" style="vertical-align:middle;" />×'
                + obj.retweet_count;
    }

    // お気に入りにされた件数
    var fav_stat = "";
    if (obj.favorite_count) {
        fav_stat = '<img src="' + '{$SITE_URL}' + 'twitter/favorited.png" width="15" height="14"'
                 + ' alt="'
                 + obj.favorite_count
                 + '人のお気に入り" title="'
                 + obj.favorite_count
                 + '人のお気に入り" border="0" style="vertical-align:middle;" />×'
                 + obj.favorite_count;
    }

    // リツイート数とお気に入り数の間のスペーサー
    var spacer = "";
    if (obj.retweet_count && obj.favorite_count) {
        spacer = " ";
    }

    // 端末判別
    var ua = obj.source;
    if (ua.match(/movatwi|yubitter|Keitai/i)) {
        ua = '&nbsp;<img src="' + '{$SITE_URL}' + 'twitter/mobile.png" width="13" height="14"'
           + ' alt="携帯から投稿" title="携帯から投稿" border="0" style="vertical-align:middle;" />';
    }
    else if (ua.match(/iphone/i)) {
        ua = '&nbsp;<img src="' + '{$SITE_URL}' + 'twitter/iphone.png" width="13" height="14"'
           + ' alt="iPhoneから投稿" title="iPhoneから投稿" border="0" style="vertical-align:middle;" />';
    }
    else if (ua.match(/Echofon/i)) {
        ua = '&nbsp;<img src="' + '{$SITE_URL}' + 'twitter/iphone.png" width="13" height="14"'
           + ' alt="iPhoneから投稿" title="iPhoneから投稿" border="0" style="vertical-align:middle;" />';
    }
    else {
        ua = '&nbsp;<img src="' + '{$SITE_URL}' + 'twitter/pc.png" width="13" height="14"'
           + ' alt="PCから投稿" title="PCから投稿" border="0" style="vertical-align:middle;" />';
    }

    // 付帯情報のフォーマッティング
    var incidental = '<dd>'
                   + replyTo
                   + point
                   + '<a href="'
                   + tweetUrl
                   + '" title="このツイート（スレッド）を見る" target="_blank">'
                   + dateText
                   + '</a>'
                   + ua
                   + '<br />'
                   + rt_stat
                   + spacer
                   + fav_stat
                   + '</dd>';

    /*
     * 付帯情報の出力
     */
    document.write(incidental);
}

document.write('</dl>');

_EOM_;

    echo $output;

    /* 無意味だが念の為のおまじない */
    ob_flush(); // PHPの出力バッファを強制出力
    flush();    // Webサーバの出力バッファを強制出力

}

function getStatuses() {
/*
 * 【JSONデータ取得】
 *      機能：TwitterからタイムラインJSONを取得
 *
 *      引数：なし
 *      戻値：JSONデータ文字列オブジェクト
 */

    $json = "";

    $url = TWITTER_API . '/1.1/statuses/user_timeline.json';
    $query_string = '?screen_name=' . USER_ID . '&count=' . GET_TWEET_CNT;
    if (! GET_RETWEET) {
        $query_string .= '&include_rts=false';
        $include_rts = "false";
    }
    else {
        $query_string .= '&include_rts=true';
        $include_rts = "true";
    }

    $oauth_nonce = md5(uniqid(mt_rand(), TRUE));
    $oauth_timestamp = time();

    // Signature base string 生成用パラメータ配列
    $signature_param = array(
        'oauth_consumer_key' => TWTR_CONSUMER_KEY,
        'oauth_nonce' => $oauth_nonce,
        'oauth_signature_method' => 'HMAC-SHA1',
        'oauth_timestamp' => $oauth_timestamp,
        'oauth_token' => TWTR_ACCESS_TOKEN,
        'oauth_version' => '1.0',
        'screen_name' => USER_ID,
        'count' => GET_TWEET_CNT,
        'include_rts' => $include_rts
    );

    // Signature base string 生成用パラメータ配列をキーでソート
    uksort($signature_param, 'strnatcmp');

    // Signature base string を作成
    $signature_queries = array();
    foreach ($signature_param as $k => $v) $signature_queries[] = $k. '='. _rawurlencode($v);
    $signature_query = implode('&', $signature_queries);

    // それぞれをURLエンコードして接続
    $twtr_signature_base  = "GET&";
    $twtr_signature_base .= _rawurlencode($url). '&';
    $twtr_signature_base .= _rawurlencode($signature_query);

    // Authorization header 生成用パラメータ配列
    $header_param = array(
        'oauth_consumer_key' => TWTR_CONSUMER_KEY,
        'oauth_nonce' => $oauth_nonce,
        'oauth_signature_method' => 'HMAC-SHA1',
        'oauth_timestamp' => $oauth_timestamp,
        'oauth_token' => TWTR_ACCESS_TOKEN,
        'oauth_version' => '1.0'
    );

    // oauth_signatureを生成して$header_paramに加える
    $twtr_keys = _rawurlencode(TWTR_CONSUMER_SECRET). '&'. _rawurlencode(TWTR_ACCESS_SECRET);
    $header_param['oauth_signature'] = base64_encode(_hmacsha1($twtr_keys, $twtr_signature_base));

    // Authorization header 生成用パラメータ配列をキーでソート
    uksort($header_param, 'strnatcmp');

    // HTTPヘッダーに追加する Authorization header を作成
    $request_headers = array();
    foreach ($header_param as $k => $v) $request_headers[] = $k. '="'. _rawurlencode($v). '"';
    //$request_header = 'OAuth realm="'. TWITTER_API . '", '. implode(', ', $request_headers);
    $request_header = 'OAuth '. implode(', ', $request_headers);

    // cURLでアクセス
    $conn = curl_init();
    curl_setopt($conn, CURLOPT_URL, $url . $query_string);
    curl_setopt($conn, CURLOPT_RETURNTRANSFER, true);
    //curl_setopt($conn, CURLOPT_HEADER, true);
    curl_setopt($conn, CURLOPT_HTTPHEADER, array('Authorization: '.$request_header));
    curl_setopt($conn, CURLOPT_ENCODING, 'gzip,deflate');
    $json = curl_exec($conn);

    return ($json);
}

//URLエンコード RFC3986版
function _rawurlencode($str) {
    $str = rawurlencode($str);
    $str = str_replace('+', ' ', $str);
    $str = str_replace('%7E', '~', $str);
    return $str;
}

//hash_hmac('sha1', $data, $key, true)の代わり
function _hmacsha1($key, $data) {
    $blocksize = 64;
    $hashfunc = 'sha1';
    if (strlen($key) > $blocksize) $key = pack('H*', $hashfunc($key));
    $key = str_pad($key, $blocksize, chr(0x00));
    $ipad = str_repeat(chr(0x36), $blocksize);
    $opad = str_repeat(chr(0x5c), $blocksize);
    $hmac = pack('H*', $hashfunc(($key^$opad). pack('H*', $hashfunc(($key^$ipad). $data))));
    return $hmac;
}

?>
